#ifndef POKEPLATINUM_OV32_02256470_H
#define POKEPLATINUM_OV32_02256470_H

#include "struct_decls/struct_02018340_decl.h"
#include "overlay032/struct_ov32_02256470_1.h"
#include "overlay032/struct_ov32_02256470_decl.h"

BOOL ov32_02256470(UnkStruct_ov32_02256470 ** param0, const UnkStruct_ov32_02256470_1 * param1, UnkStruct_02018340 * param2);
void ov32_02256508(UnkStruct_ov32_02256470 * param0);
void ov32_02256538(UnkStruct_ov32_02256470 * param0, u32 param1);
BOOL ov32_0225655C(UnkStruct_ov32_02256470 * param0, u32 param1);
BOOL ov32_02256568(UnkStruct_ov32_02256470 * param0);
u32 ov32_02256B78(u32 param0, u32 param1, u32 param2);

#endif // POKEPLATINUM_OV32_02256470_H
